@@ -1,5 +1,5 @@
 package PAR;
-$PAR::VERSION = '0.981_01';
+$PAR::VERSION = '0.985_01';
 
 use 5.006;
 use strict;
@@ -35,7 +35,7 @@ PAR - Perl Archive Toolkit
 
 =head1 VERSION
 
-This document describes development release 0.981_01 of PAR, released August 10, 2008.
+This document describes release 0.985_01 of PAR, released February  2, 2008.
 
 =head1 SYNOPSIS
 
@@ -135,6 +135,13 @@ If you specify the C<install =E<gt> 1> option in the C<use PAR>
 line above, the distribution containing C<Module> will be permanently
 installed on your system. (C<use PAR { repository =E<gt> 'http://foo/bar', install =E<gt> 1 };>)
 
+Furthermore, there is an C<upgrade =E<gt> 1> option that checks for upgrades
+in the repository in addition to installing. Please note that an upgraded
+version of a module is only loaded on the next run of your application.
+
+Adding the C<dependencies =E<gt> 1> option will enable PAR::Repository::Client's
+static dependency resolution (PAR::Repository::Client 0.23 and up).
+
 Finally, you can combine the C<run> and C<repository>
 options to run an application directly from a repository! (And you can add
 the C<install> option, too.)
@@ -305,6 +312,8 @@ use vars qw(@LibCache %LibCache);   # I really miss pseudohash.
 use vars qw($LastAccessedPAR $LastTempFile);
 use vars qw(@RepositoryObjects);    # If we have PAR::Repository::Client support, we
                                     # put the ::Client objects in here.
+use vars qw(@UpgradeRepositoryObjects); # If we have PAR::Repository::Client's in upgrade mode
+                                        # put the ::Client objects in here *as well*.
 use vars qw(%FileCache);            # The Zip-file file-name-cache
                                     # Layout:
                                     # $FileCache{$ZipObj}{$FileName} = $Member
@@ -333,6 +342,10 @@ sub import {
 
     my @args = @_;
     
+    # Insert PAR hook in @INC.
+    unshift @INC, \&find_par   unless grep { $_ eq \&find_par }      @INC;
+    push @INC, \&find_par_last unless grep { $_ eq \&find_par_last } @INC;
+
     # process args to use PAR 'foo.par', { opts }, ...;
     foreach my $par (@args) {
         if (ref($par) eq 'HASH') {
@@ -355,10 +368,6 @@ sub import {
     return if $PAR::__import;
     local $PAR::__import = 1;
 
-    # Insert PAR hook in @INC.
-    unshift @INC, \&find_par   unless grep { $_ eq \&find_par }      @INC;
-    push @INC, \&find_par_last unless grep { $_ eq \&find_par_last } @INC;
-
     require PAR::Heavy;
     PAR::Heavy::_init_dynaloader();
 
@@ -487,7 +496,7 @@ sub _import_hash_ref {
             croak("Cannot run script '$script' from PAR file '$opt->{file}'. Script couldn't be found in PAR file.");
         }
         
-        _run_member($member);
+        _run_member_from_par($member);
     }
 
     return();
@@ -502,9 +511,17 @@ sub _import_repository {
     my $url = $opt->{repository};
 
     eval "require PAR::Repository::Client; 1;";
-    if ($@ or not PAR::Repository::Client->VERSION >= 0.04) {
+    if ($@ or not eval PAR::Repository::Client->VERSION >= 0.04) {
         croak "In order to use the 'use PAR { repository => 'url' };' syntax, you need to install the PAR::Repository::Client module (version 0.04 or later) from CPAN. This module does not seem to be installed as indicated by the following error message: $@";
     }
+    
+    if ($opt->{upgrade} and not eval PAR::Repository::Client->VERSION >= 0.22) {
+        croak "In order to use the 'upgrade' option, you need to install the PAR::Repository::Client module (version 0.22 or later) from CPAN";
+    }
+
+    if ($opt->{dependencies} and not eval PAR::Repository::Client->VERSION >= 0.23) {
+        croak "In order to use the 'dependencies' option, you need to install the PAR::Repository::Client module (version 0.23 or later) from CPAN";
+    }
 
     my $obj;
 
@@ -514,12 +531,17 @@ sub _import_repository {
     }
     else {
         $obj = PAR::Repository::Client->new(
-            uri => $url,
-            auto_install => $opt->{install},
+            uri                 => $url,
+            auto_install        => $opt->{install},
+            auto_upgrade        => $opt->{upgrade},
+            static_dependencies => $opt->{dependencies},
         );
     }
 
     push @RepositoryObjects, $obj;
+    # these are tracked separately so we can check for upgrades early
+    push @UpgradeRepositoryObjects, $obj if $opt->{upgrade};
+
     return $obj;
 }
 
@@ -536,6 +558,27 @@ sub _first_member {
     return;
 }
 
+sub _run_member_from_par {
+    my $member = shift;
+    my $clear_stack = shift;
+    my ($fh, $is_new, $filename) = _tempfile($member->crc32String . ".pl");
+
+    if ($is_new) {
+        my $file = $member->fileName;
+        print $fh "package main;\n";
+        print $fh "#line 1 \"$file\"\n";
+        $member->extractToFileHandle($fh);
+        seek ($fh, 0, 0);
+    }
+
+    $ENV{PAR_0} = $filename; # for Pod::Usage
+    { do $filename;
+      CORE::exit($1) if ($@ =~/^_TK_EXIT_\((\d+)\)/);
+      die $@ if $@;
+      exit;
+    }
+}
+
 sub _run_member {
     my $member = shift;
     my $clear_stack = shift;
@@ -562,39 +605,63 @@ sub _run_member {
     }
 }
 
+# extract the contents of a .par (or .exe) or any
+# Archive::Zip handle to the PAR_TEMP/inc directory.
+# returns that directory.
 sub _extract_inc {
-    my $file = shift;
+    my $file_or_azip_handle = shift;
+    my $force_extract = shift;
     my $inc = "$PAR::SetupTemp::PARTemp/inc";
     my $dlext = defined($Config{dlext}) ? $Config::Config{dlext} : '';
+    my $inc_exists = -d $inc;
+    my $is_handle = ref($file_or_azip_handle) && $file_or_azip_handle->isa('Archive::Zip::Archive');
 
-    if (!-d $inc) {
+    require File::Spec;
+
+    if (!$inc_exists or $force_extract) {
         for (1 .. 10) { mkdir("$inc.lock", 0755) and last; sleep 1 }
         
-        # First try to unzip the *fast* way.
-        eval {
-          require Archive::Unzip::Burst;
-          Archive::Unzip::Burst::unzip($file, $inc)
-            and die "Could not unzip '$file' into '$inc'. Error: $!";
-        };
+        undef $@;
+        if (!$is_handle) {
+          # First try to unzip the *fast* way.
+          eval {
+            require Archive::Unzip::Burst;
+            Archive::Unzip::Burst::unzip($file_or_azip_handle, $inc)
+              and die "Could not unzip '$file_or_azip_handle' into '$inc'. Error: $!";
+              die;
+          };
 
-        # This means the fast module is there, but didn't work.
-        if ($@ =~ /^Could not unzip/) {
-          die $@;
+          # This means the fast module is there, but didn't work.
+          if ($@ =~ /^Could not unzip/) {
+            die $@;
+          }
         }
 
-        # failed to load Archive::Unzip::Burst. Default to slow way.
-        elsif ($@) {
-          open my $fh, '<', $file or die "Cannot find '$file': $!";
-          binmode($fh);
-          bless($fh, 'IO::File');
+        # either failed to load Archive::Unzip::Burst or got an A::Zip handle
+        # fallback to slow way.
+        if ($is_handle || $@) {
+          my $zip;
+          if (!$is_handle) {
+            open my $fh, '<', $file_or_azip_handle
+              or die "Cannot find '$file_or_azip_handle': $!";
+            binmode($fh);
+            bless($fh, 'IO::File');
 
-          my $zip = Archive::Zip->new;
-          ( $zip->readFromFileHandle($fh, $file) == Archive::Zip::AZ_OK() )
-              or die "Read '$file' error: $!";
+            $zip = Archive::Zip->new;
+            ( $zip->readFromFileHandle($fh, $file_or_azip_handle) == Archive::Zip::AZ_OK() )
+                or die "Read '$file_or_azip_handle' error: $!";
+          }
+          else {
+            $zip = $file_or_azip_handle;
+          }
+
+          mkdir($inc) if not -d $inc;
 
           for ( $zip->memberNames() ) {
               next if m{\.\Q$dlext\E[^/]*$};
               s{^/}{};
+              my $outfile =  File::Spec->catfile($inc, $_);
+              next if -e $outfile and not -w _;
               $zip->extractMember($_, "$inc/" . $_);
           }
         }
@@ -602,16 +669,51 @@ sub _extract_inc {
         rmdir("$inc.lock");
     }
 
-    require File::Spec;
-    unshift @INC, grep -d, map File::Spec->catdir($inc, @$_),
-        [ 'lib' ], [ 'arch' ], [ $arch ], [ $ver ], [ $ver, $arch ], [];
+    # add the freshly extracted directories to @INC,
+    # but make sure there's no duplicates
+    my %inc_exists = map { ($_, 1) } @INC;
+    unshift @INC, grep !exists($inc_exists{$_}),
+                  grep -d,
+                  map File::Spec->catdir($inc, @$_),
+                  [ 'lib' ], [ 'arch' ], [ $arch ],
+                  [ $ver ], [ $ver, $arch ], [];
+
+    return $inc;
 }
 
 
 # This is the hook placed in @INC for loading PAR's
 # before any other stuff in @INC
 sub find_par {
-    return _find_par_internals(\@PAR_INC, @_);
+    my @args = @_;
+
+    # if there are repositories win upgrade mode, check them
+    # first. If so, this is expensive, of course!
+    if (@UpgradeRepositoryObjects) {
+        my $module = $args[1];
+        $module =~ s/\.pm$//;
+        $module =~ s/\//::/g;
+        foreach my $client (@UpgradeRepositoryObjects) {
+            my $local_file = $client->upgrade_module($module);
+
+            # break the require if upgrade_module has been required already
+            # to avoid infinite recursion
+            if (exists $INC{$args[1]}) {
+                # Oh dear. Check for the possible return values of the INC sub hooks in
+                # perldoc -f require before trying to understand this.
+                # Then, realize that if you pass undef for the file handle, perl (5.8.9)
+                # does NOT use the subroutine. Thus the hacky GLOB ref.
+                my $line = 1;
+                return (\*I_AM_NOT_HERE, sub {$line ? ($_="1;",$line=0,return(1)) : ($_="",return(0))});
+            }
+
+            if ($local_file) {
+                return _find_par_internals([$PAR_INC_LAST[-1]], @args);
+            }
+        }
+    }
+
+    return _find_par_internals(\@PAR_INC, @args);
 }
 
 # This is the hook placed in @INC for loading PAR's
@@ -631,7 +733,7 @@ sub find_par_last {
     $module =~ s/\.pm$//;
     $module =~ s/\//::/g;
     foreach my $client (@RepositoryObjects) {
-        my $local_file = $client->get_module($module, 1);
+        my $local_file = $client->get_module($module, 1); # 1 == fallback
         if ($local_file) {
             return _find_par_internals([$PAR_INC_LAST[-1]], @args);
         }
@@ -997,6 +1099,9 @@ L<Archive::Zip>, L<perlfunc/require>
 
 L<ex::lib::zip>, L<Acme::use::strict::with::pride>
 
+Steffen Mueller has detailed slides on using PAR for application
+deployment at L<http://steffen-mueller.net/talks/appdeployment/>.
+
 PAR supports the L<prefork> module. It declares various run-time
 dependencies so you can use the L<prefork> module to get streamlined
 processes in a forking environment.
@@ -1022,6 +1127,8 @@ have sent helpful patches, ideas or comments.
 
 Audrey Tang E<lt>cpan@audreyt.orgE<gt>
 
+Steffen Mueller E<lt>smueller@cpan.orgE<gt>
+
 L<http://par.perl.org/> is the official PAR website.  You can write
 to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
 E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
@@ -1032,8 +1139,9 @@ preferred.
 
 =head1 COPYRIGHT
 
-Copyright 2002-2008 by Audrey Tang
+Copyright 2002-2009 by Audrey Tang
 E<lt>cpan@audreyt.orgE<gt>.
+Copyright 2005-2009 by Steffen Mueller E<lt>smueller@cpan.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
